/*
   Grant Plaster - ObjMeshLoaderSimple.h
   Helper file for ObjMeshLoaderSimple.cpp. Based off of code provided in
   CMeshLoaderSimple.h by Zoe Wood.
*/

#ifndef _OBJMESHLOADER_H_INCLUDED_
#define _OBJMESHLOADER_H_INCLUDED_

#include <string>

#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif

#ifdef __unix__
#include <GL/glut.h>
#endif

#ifdef _WIN32
#include <GL\glew.h>
#include <GL\glut.h>
#endif


class OMesh;

class OMeshLoader
{

	/*!
	 * Attempts to load a mesh from an ascii .obj file
	 * \return A pointer to a mesh object or NULL if the mesh failed to load
	 */
	static OMesh * const loadASCIIMesh(std::string const & fileName);

	/*!
	 * Generates a VBO from a given mesh file.
	 * \param Mesh The mesh to generate the VBO from.
	 * \param TriangleCount Returns the number of triangles to render from the VBO
	 * \param PositionBufferHandle Returns a handle to the position buffer object
	 * \param ColorBufferHandle Returns a handle to the color buffer object
	 */
	static void createVertexBufferObject(OMesh const & Mesh, int & TriangleCount, GLuint & PositionBufferHandle, GLuint & ColorBufferHandle, GLuint & NormalBufferHandle, GLuint & UVBufferHandle, GLuint & MatBufferHandle);

public:

	static bool loadVertexBufferObjectFromMesh(std::string const & fileName, int & TriangleCount, GLuint & PositionBufferHandle, GLuint & ColorBufferHandle, GLuint & NormalBufferHandle, GLuint & UVBufferHandle, GLuint & MatBufferHandle);

};

#endif
