/*
  Base code for program 3 for CSC 471
  OpenGL, glut and GLSL application
  Loads in a .m mesh file into a VBO and draws it
  Uses glm for matrix transforms
  I. Dunn and Z. Wood  (original .m loader by H. Hoppe)
*/

#include <iostream>
#include <fstream>
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif

#ifdef __unix__
#include <GL/glut.h>
#endif

#ifdef _WIN32
#pragma comment(lib, "glew32.lib")

#include <GL\glew.h>
#include <GL\glut.h>
#endif

#include "CMeshLoaderSimple.h"
#include "ObjMeshLoaderSimple.h"
#include "GeometryCreator.h"
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "mesh.h"
#include "GLSL_helper.h"
#include "MStackHelp.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr

#define PI 3.1415926535897
#define numTrees 40

using namespace std;

//TEXTURE STUFF
/*data structure for the image used for  texture mapping */
typedef struct Image {
  unsigned long sizeX;
  unsigned long sizeY;
  char *data;
} Image;

Image *TextureImage;

typedef struct RGB {
  GLubyte r;
  GLubyte g; 
  GLubyte b;
} RGB;

RGB myimage[64][64];
RGB* g_pixel;


//forward declaration of image loading and texture set-up code
int ImageLoad(char *filename, Image *image);
GLvoid LoadTexture(char* image_file, int tex_id);

void drawMainModel(void);
void animate(void);

//position and color data handles
GLuint triBuffObj, colBuffObj, normalBuffObj;
GLuint triBuffObj2, colBuffObj2, normalBuffObj2;

//GLuint iboH, iboN, iboC, iboS, iboUL, iboLL, iboUA, iboLA, iboFoot, iboHand;
GLuint posH, posN, posC, posS, posUL, posLL, posUA, posLA, posFoot, posHand, posTree, posSword;
GLuint colH, colN, colC, colS, colUL, colLL, colUA, colLA, colFoot, colHand, colTree, colSword;
GLuint normH, normN, normC, normS, normUL, normLL, normUA, normLA, normFoot, normHand, normTree, normSword;
GLuint uvsH, uvsN, uvsC, uvsS, uvsUL, uvsLL, uvsUA, uvsLA, uvsFoot, uvsHand, uvsTree, uvsSword;
GLuint matH, matN, matC, matS, matUL, matLL, matUA, matLA, matFoot, matHand, matTree, matSword;
int vertsH, vertsN, vertsC, vertsS, vertsUL, vertsLL, vertsUA, vertsLA, vertsFoot, vertsHand, vertsTree, vertsSword;

//flag and ID to toggle on and off the shader
int shade = 1;
static int nasty = 2;
static int mat = 0;
int ShadeProg;
int TriangleCount, TriangleCount2;
static float g_width, g_height;
float g_Camtrans = -2.5;
float g_Camangle = 0;
int g_mat_id = 0;
glm::vec3 g_trans(0);

//int numTrees = 30;

//MODEL FLOATS
float mainZ = 2.0;
float BSx = 3.0, BSy = 5.0;
//float BSx = 1 / 16.0, BSy = 1 / 16.0, BSz = 1 / 16.0;
//float ASx = 1.25, ASy = 3.5;
float ASx = 1.25, ASy = 3.5;
float LSx = 1.5, LSy = 4.0;

//SCALE TO ONE
float BSOx = 1.5, BSOy = 1.25, BSOz = 3.0;
float ASOx = 2.0, ASOy = 3.0, ASOz = 1.5;
float ASOLx = 2.5, ASOLy = 1.5, ASOLz = 2.5;
float HSOx = 3.0, HSOy = 2.5, HSOz = 3.0;
float LSOx = 1.75, LSOy = 1.5, LSOz = 1.75;
float LSOLx = 3.5, LSOLy = 2.0, LSOLz = 3.5;
float FSOx = 4.0, FSOy = 5.0, FSOz = 2.5;
float NSOx = 6.0, NSOy = 16.0, NSOz = 6.0;
float HeSOx = 3.0, HeSOy = 3.0, HeSOz = 3.0;
//TRANS TO ORIGIN
float ATOx = -0.25, ATOy = -0.25, ATOz = 0.0;
float ATOLx = -0.35, ATOLy = -0.05, ATOLz = -0.10;
float HTOx = -0.35, HTOy = -0.10, HTOz = 0.0;
float LTOx = -0.10, LTOy = 0.15, LTOz = 0.0;
float LTOLx = -0.05, LTOLy = 0.25, LTOLz = -0.05;
float FTOx = -0.05, FTOy = 0.25, FTOz = -0.10;
float NTOx = 0.0, NTOy = -0.425, NTOz = -0.10;
float HeTOx = 0.0, HeTOy = -0.40, HeTOz = -0.15;

//ANIMATION FLAGS
char mainState = 's';
char backState = 'w';
char moveCase = 'n';
char turnCase = 'n';
bool wDown = false, sDown = false, aDown = false, dDown = false, jDown = false, lDown = false, kDown = false, iDown = false;

//STAND
bool standUp = false;
//WALK
bool leftArmForward = false;
//RUN
bool rightArmForward = true;

float posDirx = 0.0;
float posDirz = 0.1;
float posAng = 0.0;
float radP = 0.0;

//VIEW
char camState = 'n';

//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//ANIMATION ANGS
struct bodAng {
   float x;
   float y;
   float z;
} ;
bodAng origPos = {0.0, 11.0, 0.0};
//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//CURRENT
//bodAng curARU, curALU, curARL, curALL, curLRU, curLLU, curLRL, curLLL, curT, curH, curN, curLH, curRH, curLF, curRF;
//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//PHYSICAL CAPABILITY
/*bodAng minT = {-20.0, -70.0, -20.0}, maxT = {90.0, 70.0, 20.0};                                                             // TORSO
bodAng minARU = {-180.0, 0.0, 0.0}, maxARU = {75.0, 0.0, 100.0}, minALU = {-180.0, 0.0, -100.0}, maxALU = {75.0, 0.0, 0.0};   // UPPER ARMS
bodAng minARL = {-160.0, 0.0, -160.0}, maxARL = {0.0, 0.0, 0.0}, minALL = {-160.0, 0.0, 0.0}, maxALL = {0.0, 0.0, 160.0};     // LOWER ARMS
bodAng minLRU = {-160.0, 0.0, 0.0}, maxLRU = {50.0, 0.0, 60.0}, minLLU = {-160.0, 0.0, -60.0}, maxLLU = {50.0, 0.0, 0.0};     // UPPER LEGS
bodAng minLRL = {0.0, 0.0, -40.0}, maxLRL = {160.0, 0.0, 40.0}, minLLL = {0.0, 0.0, -40.0}, maxLLL = {160.0, 0.0, 40.0};      // LOWER LEGS
bodAng minH = {-45.0, -45.0, -45.0}, maxH = {45.0, -45.0, 45.0};                                                              // HEAD
bodAng minN = {-45.0, -45.0, -45.0}, maxN = {90.0, 45.0, 45.0};                                                               // NECK
bodAng minRH = {-35.0, -90.0, 90.0}, maxRH = {60.0, 90.0, 90.0}, minLH = {-35.0, -90.0, -90.0}, maxLH = {60.0, 90.0, 90.0};   // HANDS
bodAng minRF = {-35.0, 0.0, -45.0}, maxRF = {55.0, 0.0, 45.0}, minLF = {-35.0, 0.0, -45.0}, maxLF = {55.0, 0.0, 45.0};*/      // FEET
//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//STANDING ANGS
bodAng fromST = {0.0, 0.0, 0.0}, toST = {0.0, 0.0, 0.0};
bodAng fromSARU = {5.0, 0.0, -10.0}, toSARU = {5.0, 0.0, -10.0}, fromSALU = {0.0, 0.0, 20.0}, toSALU = {0.0, 0.0, 20.0};
bodAng fromSARL = {-2.0 * fromSARU.x, 0.0, -2.0 * fromSARU.z}, toSARL = {-2.0 * toSARU.x, 0.0, -2.0 * toSARU.z}, 
       fromSALL = {0.0, 0.0, -fromSALU.z}, toSALL = {0.0, 0.0, -toSALU.z};
bodAng fromSLRU = {-5.0, 0.0, -5.0}, toSLRU = {-15.0 - toST.x, 0.0, -5.0}, fromSLLU = {-5.0, 0.0, 5.0}, toSLLU = {-15.0 - toST.x, 0.0, 5.0};
bodAng fromSLRL = {-fromSLRU.x, 0.0, -fromSLRU.z}, toSLRL = {-2.0 * toSLRU.x - toST.x, 0.0, -toSLRU.z}, 
       fromSLLL = {-fromSLLU.x, 0.0, -fromSLLU.z}, toSLLL = {-2.0 * toSLLU.x - toST.x, 0.0, -toSLLU.z};
bodAng fromSH = {-10.0, 0.0, 0.0};//, toSH = {0.0, 0.0, 0.0};
bodAng fromSN = {0.0, 0.0, 0.0};//, toSN = {0.0, 0.0, 0.0};
bodAng fromSRH = {0.0, 0.0, 0.0}, toSRH = {0.0, 0.0, 0.0}, fromSLH = {15.0, -8.0, 0.0}, toSLH = {10.0, -14.0, 0.0};
bodAng fromSRF = {-fromSLRU.x - fromSLRL.x - fromST.x, -fromSLRU.y - fromSLRL.y - fromST.y, -fromSLRU.z - fromSLRL.z - fromST.z},
       toSRF = {-toSLRU.x - toSLRL.x - toST.x, -toSLRU.y - toSLRL.y - toST.y, -toSLRU.z - toSLRL.z - toST.z},
       fromSLF = {-fromSLLU.x - fromSLLL.x - fromST.x, -fromSLLU.y - fromSLLL.y - fromST.y, -fromSLLU.z - fromSLLL.z - fromST.z},
       toSLF = {-toSLLU.x - toSLLL.x - toST.x, -toSLLU.y - toSLLL.y - toST.y, -toSLLU.z - toSLLL.z - toST.z};
bodAng standPos = {0.0, 9.0 - (4.0 - ((cos(fromSLLU.x * PI / 180.0)) * 4.0)), 9.0 - (4.0 - ((sin(fromSLLU.x * PI / 180.0)) * 4.0))};
//------------------------------------------------------------------------------------------------------------------------------
//MODEL STARTS FROM STANDING
bodAng curARU = {fromSARU.x, fromSARU.y, fromSARU.z};
bodAng curALU = {fromSALU.x, fromSALU.y, fromSALU.z};
bodAng curARL = {fromSARL.x, fromSARL.y, fromSARL.z};
bodAng curALL = {fromSALL.x, fromSALL.y, fromSALL.z};
bodAng curLRU = {fromSLRU.x, fromSLRU.y, fromSLRU.z};
bodAng curLLU = {fromSLLU.x, fromSLLU.y, fromSLLU.z};
bodAng curLRL = {fromSLRL.x, fromSLRL.y, fromSLRL.z};
bodAng curLLL = {fromSLLL.x, fromSLLL.y, fromSLLL.z};
bodAng curT = {fromST.x, fromST.y, fromST.z};
bodAng curH = {fromSH.x, fromSH.y, fromSH.z};
bodAng curN = {fromSN.x, fromSN.y, fromSN.z};
bodAng curRH = {fromSRH.x, fromSRH.y, fromSRH.z};
bodAng curLH = {fromSLH.x, fromSLH.y, fromSLH.z};
bodAng curRF = {fromSRF.x, fromSRF.y, fromSRF.z};
bodAng curLF = {fromSLF.x, fromSLF.y, fromSLF.z};

//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//WALKING ANGS

bodAng fromWT = {0.0, 0.0, 0.0}, toWT = {0.0, 0.0, 0.0};
bodAng fromWARU = {20.0, 0.0, -10.0}, toWARU = {-20.0, 0.0, -10.0}, fromWALU = {-5.0, 0.0, 20.0}, toWALU = {5.0, 0.0, 20.0};
bodAng fromWARL = {-2.0 * fromWARU.x, 0.0, -2.0 * fromWARU.z}, toWARL = {-2.0 * toWARU.x, 0.0, -2.0 * toWARU.z}, 
       fromWALL = {-fromWALU.x, 0.0, -fromWALU.z}, toWALL = {-toWALU.x, 0.0, -toWALU.z};
bodAng fromWLRU = {-22.5, 0.0, -5.0}, toWLRU = {22.5, 0.0, -5.0}, fromWLLU = {22.5, 0.0, 5.0}, toWLLU = {-22.5, 0.0, 5.0};
bodAng fromWLRL = {-fromWLRU.x - 15.0, 0.0, -fromWLRU.z}, toWLRL = {toWLRU.x - 15.0, 0.0, -toWLRU.z}, 
       fromWLLL = {fromWLLU.x - 15.0, 0.0, -fromWLLU.z}, toWLLL = {-toWLLU.x - 15.0, 0.0, -toWLLU.z};
bodAng fromWH = {0.0, 0.0, 0.0};//, toWH = {0.0, 0.0, 0.0};
bodAng fromWN = {0.0, 0.0, 0.0};//, toWN = {0.0, 0.0, 0.0};
bodAng fromWRH = {0.0, 0.0, 0.0}, toWRH = {0.0, 0.0, 0.0}, fromWLH = {0.0, 35.0, 0.0}, toWLH = {0.0, 70.0, 0.0};
bodAng fromWRF = {-fromWLRU.x - fromWLRL.x, -fromWLRU.y - fromWLRL.y, -fromWLRU.z - fromWLRL.z},
       toWRF = {-toWLRU.x - toWLRL.x, -toWLRU.y - toWLRL.y, -toWLRU.z - toWLRL.z},
       fromWLF = {-fromWLLU.x - fromWLLL.x, -fromWLLU.y - fromWLLL.y, -fromWLLU.z - fromWLLL.z},
       toWLF = {-toWLLU.x - toWLLL.x, -toWLLU.y - toWLLL.y, -toWLLU.z - toWLLL.z};
float midWLx, midWRx;
//standPos = {0.0, 11.0 - (4.0 - ((cos(fromWLLU.x * PI / 180.0)) * 4.0)), 11.0 - (4.0 - ((sin(fromWLLU.x * PI / 180.0)) * 4.0))};

/*bodAng curARU = {fromWARU.x, fromWARU.y, fromWARU.z};
bodAng curALU = {fromWALU.x, fromWALU.y, fromWALU.z};
bodAng curARL = {fromWARL.x, fromWARL.y, fromWARL.z};
bodAng curALL = {fromWALL.x, fromWALL.y, fromWALL.z};
bodAng curLRU = {fromWLRU.x, fromWLRU.y, fromWLRU.z};
bodAng curLLU = {fromWLLU.x, fromWLLU.y, fromWLLU.z};
bodAng curLRL = {fromWLRL.x, fromWLRL.y, fromWLRL.z};
bodAng curLLL = {fromWLLL.x, fromWLLL.y, fromWLLL.z};
bodAng curT = {fromWT.x, fromWT.y, fromWT.z};
bodAng curH = {fromWH.x, fromWH.y, fromWH.z};
bodAng curN = {fromWN.x, fromWN.y, fromWN.z};
bodAng curRH = {fromWRH.x, fromWRH.y, fromWRH.z};
bodAng curLH = {fromWLH.x, fromWLH.y, fromWLH.z};
bodAng curRF = {fromWRF.x, fromWRF.y, fromWRF.z};
bodAng curLF = {fromWLF.x, fromWLF.y, fromWLF.z};*/

//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------
//RUNNING ANGS

bodAng fromRT = {40.0, 0.0, 0.0}, toRT = {50.0, 0.0, 0.0};
bodAng fromRARU = {45.0, 0.0, -10.0}, toRARU = {-60.0, 0.0, -10.0}, fromRALU = {25.0, 0.0, 5.0}, toRALU = {30.0, 0.0, 5.0};
bodAng fromRARL = {-90.0, 0.0, 0.0}, toRARL = {-90, 0.0, 0.0}, 
       fromRALL = {5.0, 0.0, 15.0}, toRALL = {5.0, 0.0, 15.0};
bodAng fromRLRU = {-80.0 - fromRT.x, 0.0, -5.0}, toRLRU = {40.0 - fromRT.x, 0.0, -5.0}, fromRLLU = {40.0 - fromRT.x, 0.0, 5.0}, toRLLU = {-80.0 - fromRT.x, 0.0, 5.0};
bodAng fromRLRL = {135.0, 0.0, -fromRLRU.z}, midRLRL = {160.0, 0.0, 0.0}, toRLRL = {5.0, 0.0, -toRLRU.z}, 
       fromRLLL = {5.0, 0.0, -fromRLLU.z}, midRLLL = {160.0, 0.0, -fromRLLU.z}, toRLLL = {135.0, 0.0, -toRLLU.z};
bodAng fromRH = {-20.0, 0.0, 0.0}, toRH = {-20.0, 0.0, 0.0};
bodAng fromRN = {-20.0, 0.0, 0.0}, toRN = {-20.0, 0.0, 0.0};
bodAng fromRRH = {0.0, 0.0, 0.0}, toRRH = {0.0, 0.0, 0.0}, fromRLH = {-45.0, 140.0, 0.0}, toRLH = {-35.0, 150.0, 0.0};
bodAng fromRRF = {-fromRLRU.x - fromRLRL.x - toRT.x, -fromRLRU.y - fromRLRL.y, -fromRLRU.z - fromRLRL.z},
       toRRF = {-35.0, -toRLRU.y - toRLRL.y, -toRLRU.z - toRLRL.z},
       fromRLF = {-35.0, -fromRLLU.y - fromRLLL.y, -fromRLLU.z - fromRLLL.z},
       toRLF = {-toRLLU.x - toRLLL.x - toRT.x, -toRLLU.y - toRLLL.y, -toRLLU.z - toRLLL.z};
float midRLx, midRRx;
float midRLFx, midRRFx;
//standPos = {0.0, 11.0 - (4.0 - ((cos(fromRLLU.x * PI / 180.0)) * 4.0)), 11.0 - (4.0 - ((sin(fromRLLU.x * PI / 180.0)) * 4.0))};

/*bodAng curARU = {fromRARU.x, fromRARU.y, fromRARU.z};
bodAng curALU = {fromRALU.x, fromRALU.y, fromRALU.z};
bodAng curARL = {fromRARL.x, fromRARL.y, fromRARL.z};
bodAng curALL = {fromRALL.x, fromRALL.y, fromRALL.z};
bodAng curLRU = {fromRLRU.x, fromRLRU.y, fromRLRU.z};
bodAng curLLU = {fromRLLU.x, fromRLLU.y, fromRLLU.z};
bodAng curLRL = {fromRLRL.x, fromRLRL.y, fromRLRL.z};
bodAng curLLL = {fromRLLL.x, fromRLLL.y, fromRLLL.z};
bodAng curT = {fromRT.x, fromRT.y, fromRT.z};
bodAng curH = {fromRH.x, fromRH.y, fromRH.z};
bodAng curN = {fromRN.x, fromRN.y, fromRN.z};
bodAng curRH = {fromRRH.x, fromRRH.y, fromRRH.z};
bodAng curLH = {fromRLH.x, fromRLH.y, fromRLH.z};
bodAng curRF = {fromRRF.x, fromRRF.y, fromRRF.z};
bodAng curLF = {fromRLF.x, fromRLF.y, fromRLF.z};*/

//------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------

float standAL = 20.0, standAR = -20.0;
float standALL = -20.0, standARL = 0.0;
float standLL = -5.0, standLR = 5.0;
float standLLL = 5.0, standLRL = -5.0;

bool stopB = false;

float alpha = 0;
float beta = PI / 2;

glm::vec3 camPos = glm::vec3(0.0f, 3.0f, -4.0f);
glm::vec3 LookAtPt = glm::vec3(0.0f, 0.0f, 0.0f);
glm::vec3 Up = glm::vec3(0.0f, 1.0f, 0.0f);
glm::vec3 w = glm::vec3(0.0f, 0.0f, 0.0f);
glm::vec3 u = glm::vec3(0.0f, 0.0f, 0.0f);
glm::vec3 v = glm::vec3(0.0f, 0.0f, 0.0f);

// Meshes
unsigned int const MeshCount = 14;
Mesh * Meshes[MeshCount];
unsigned int CurrentMesh;

//Handles to the shader data
GLint h_aPosition;
GLint h_aNormal;
GLint h_uModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;

GLint h_uLightPos;
GLint h_uLightColor;
GLint h_uNasty;
GLint h_uTexUnit;
GLint h_aTexCoord;
GLint h_uTexUnit2;

GLuint CubeBuffObj,CIndxBuffObj, GrndBuffObj, GIndxBuffObj, GNBuffObj, GNIndxBuffObj, RIndxBuffObj, NormalBuffObj, TexBuffObj, TexBuffObj2;
int g_CiboLen, g_GiboLen;

GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;

GLint h_uViewInv;
GLint h_uModInvTransp;

static const float g_groundY = -.51;      // y coordinate of the ground
static const float g_groundSize = 6.0;   // half the ground length

float treeTransX[numTrees] = {0.0};
float treeTransZ[numTrees] = {0.0};
float treeRad = 1.0;

RenderingHelper ModelTrans;

/* helper function to set up material for shading */
void SetMaterial(int i) {
  glUseProgram(ShadeProg);
  switch (i) {
    case 0:
        safe_glUniform3f(h_uMatAmb, 0.1, 0.3, 0.1);
        safe_glUniform3f(h_uMatDif, 0.0, 0.5, 0.08);
        safe_glUniform3f(h_uMatSpec, 0.2, 0.2, 0.2);
        safe_glUniform1f(h_uMatShine, 20.0);
        break;
    case 1:
        safe_glUniform3f(h_uMatAmb, 0.2, 0.1, 0.05);
        safe_glUniform3f(h_uMatDif, 0.6, 0.3, 0.1);
        safe_glUniform3f(h_uMatSpec, 0.3, 0.1, 0.1);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 2:
        safe_glUniform3f(h_uMatAmb, 0.3, 0.3, 0.6);
        safe_glUniform3f(h_uMatDif, 0.2, 0.6, 0.9);
        safe_glUniform3f(h_uMatSpec, 0.2, 0.5, 0.8);
        safe_glUniform1f(h_uMatShine, 1000.0);
        break;
    case 3:
        safe_glUniform3f(h_uMatAmb, 0.2, 0.1, 0.0);
        safe_glUniform3f(h_uMatDif, 0.4, 0.2, 0.1);
        safe_glUniform3f(h_uMatSpec, 0.1, 0.1, 0.0);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 4:
        safe_glUniform3f(h_uMatAmb, 0.5, 0.4, 0.0);
        safe_glUniform3f(h_uMatDif, 0.8, 0.7, 0.3);
        safe_glUniform3f(h_uMatSpec, 0.4, 0.3, 0.2);
        safe_glUniform1f(h_uMatShine, 100.0);
        break;
    case 5:
        safe_glUniform3f(h_uMatAmb, 0.7, 0.7, 0.7);
        safe_glUniform3f(h_uMatDif, 0.9, 0.9, 0.9);
        safe_glUniform3f(h_uMatSpec, 0.8, 0.6, 0.8);
        safe_glUniform1f(h_uMatShine, 1000.0);
        break;
    case 6:
        safe_glUniform3f(h_uMatAmb, 0.4, 0.3, 0.1);
        safe_glUniform3f(h_uMatDif, 1.0, 0.2, 0.3);
        safe_glUniform3f(h_uMatSpec, 0.9, 0.3, 0.5);
        safe_glUniform1f(h_uMatShine, 500.0);
        break;
    case 7:
        safe_glUniform3f(h_uMatAmb, 0.0, 0.4, 0.1);
        safe_glUniform3f(h_uMatDif, 0.1, 0.5, 0.2);
        safe_glUniform3f(h_uMatSpec, 0.1, 0.3, 0.1);
        safe_glUniform1f(h_uMatShine, 0.0);
        break;
  }
}
/* set the model transform to the identity */
/*void SetModelT() {
  glm::mat4 tmp = glm::mat4(1.0f);
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(tmp));
  safe_glUniformMatrix4fv(h_uModelMatrix2, glm::value_ptr(tmp));
}*/
static void initGround() {

  // A x-z plane at y = g_groundY of dimension [-g_groundSize, g_groundSize]^2
    float GrndPos[] = { 
    -g_groundSize, g_groundY, -g_groundSize, 
    -g_groundSize, g_groundY,  g_groundSize, 
     g_groundSize, g_groundY,  g_groundSize, 
     g_groundSize, g_groundY, -g_groundSize
    };

    float GrndNorm[] = { 
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0,
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0
    };

   static GLfloat GrndTex[] = {
      0, 0,
      0, 1,
      1, 1,
      1, 0
    }; 

    unsigned short idx[] = {0, 1, 2, 0, 2, 3};
    
    g_GiboLen = 6;
    glGenBuffers(1, &GrndBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndPos), GrndPos, GL_STATIC_DRAW);

    glGenBuffers(1, &GIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &GNBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GNBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndNorm), GrndNorm, GL_STATIC_DRAW);
   
   glGenBuffers(1, &TexBuffObj2);
    glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj2);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndTex), GrndTex, GL_STATIC_DRAW);
}
/* intialize the cube data */
static void initCube() {

  float CubePos[] = {
    -5.0, -.51, -5.0, //back face 0 verts :0 
    -5.0, 50.0, -5.0,
    5.0, 50.0, -5.0,
    5.0, -.51, -5.0,
    5.0, -.51, 5.0, //right face 5 verts :4
    5.0, 50.0, 5.0,
    5.0, 50.0, -5.0,
    5.0, -.51, -5.0,
    -5.0, -.51, 5.0, //front face 4 verts :8
    -5.0, 50.0, 5.0,
    5.0, 50.0, 5.0,
    5.0, -.51, 5.0,
    -5.0, -.51, -5.0, //left face 4 verts :12
    -5.0, 50.0, -5.0,
    -5.0, 50.0, 5.0,
    -5.0, -.51, 5.0
  };

   static GLfloat CubeTex[] = {
      /*.25, 0, // back 
      .25, 1,
      0, 1,
      0, 0,
      .75, 0, // left 
      .75, 1,
      1, 1,
      1, 0,
      .5, 0, //front 
      .5, 1,
      .75, 1,
      .75, 0,
      .25, 0, //right 
      .25, 1,
      .5, 1,
      .5, 0*/
      1, 0, // back 
      1, 1,
      0, 1,
      0, 0,
      0, 0, // left 
      0, 1,
      1, 1,
      1, 0,
      0, 0, //front 
      0, 1,
      1, 1,
      1, 0,
      0, 0, //right 
      0, 1,
      1, 1,
      1, 0
    }; 
   
    unsigned short idx[] = {0, 1, 2,  2, 3, 0,  4, 5, 6, 6, 7, 4,  8, 9, 10, 10, 11, 8,  12, 13, 14, 14, 15, 12};

    g_CiboLen = 24;
    glGenBuffers(1, &CubeBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubePos), CubePos, GL_STATIC_DRAW);

    glGenBuffers(1, &CIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);

    glGenBuffers(1, &TexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubeTex), CubeTex, GL_STATIC_DRAW);
}

//State to move object
static char transState = 't';

//Current object
int curObj = 0;

//Main Point
static float NewPoint[2] = {0, 0};

//Globals for model
static float transX = 0.0, transY = 0.0, transZ = 0.0;
static float scaleX = 1.0, scaleY = 1.0, scaleZ = 1.0;
static float angleC = 0.0;
static glm::vec3 axisRot = glm::vec3(0.0, 1.0, 0.0);
static glm::mat4 rotated = glm::mat4(1.0f);

static glm::vec3 lightPosition;
static glm::vec3 lightColor;

/* initialize the geomtry (including color)
   Change file name to load a different mesh file
*/
void InitGeom() {
  Meshes[0] = GeometryCreator::CreateCube();
  Meshes[1] = GeometryCreator::CreateSphere(glm::vec3(1.5f));
  Meshes[2] = GeometryCreator::CreateCylinder(0.25f, 0.25f, 6.f, 16, 2);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_head.obj", vertsH, posH, colH, normH, uvsH, matH);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_chest.obj", vertsC, posC, colC, normC, uvsC, matC);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_neck.obj", vertsN, posN, colN, normN, uvsN, matN);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_thigh.obj", vertsUL, posUL, colUL, normUL, uvsUL, matUL);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_calf.obj", vertsLL, posLL, colLL, normLL, uvsLL, matLL);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_upperarm.obj", vertsUA, posUA, colUA, normUA, uvsUA, matUA);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_forearm.obj", vertsLA, posLA, colLA, normLA, uvsLA, matLA);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_shoulder.obj", vertsS, posS, colS, normS, uvsS, matS);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_foot.obj", vertsFoot, posFoot, colFoot, normFoot, uvsFoot, matFoot);
  OMeshLoader::loadVertexBufferObjectFromMesh("link_hand.obj", vertsHand, posHand, colHand, normHand, uvsHand, matHand);
  OMeshLoader::loadVertexBufferObjectFromMesh("tree.obj", vertsTree, posTree, colTree, normTree, uvsTree, matTree);
  //OMeshLoader::loadVertexBufferObjectFromMesh("sword.obj", vertsSword, posSword, colSword, normSword, uvsSword, matSword);
  
  //CMeshLoader::loadVertexBufferObjectFromMesh("Models/bunny10k.m", TriangleCount, triBuffObj, colBuffObj, normalBuffObj);
  //CMeshLoader::loadVertexBufferObjectFromMesh("Models/dragon10k.m", TriangleCount2, triBuffObj2, colBuffObj2, normalBuffObj2);
  initCube();
  initGround();
}

/* projection matrix */
void SetProjectionMatrix() {
  glm::mat4 Projection = glm::perspective(90.0f, (float)g_width/g_height, 0.1f, 100.f);
  safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}

/* camera controls - do not change */
void SetView() {
  LookAtPt.x = cos(alpha) * cos(beta);
  LookAtPt.y = -sin(alpha);
  LookAtPt.z = cos(alpha) * cos((PI / 2.0) - beta);
  //LookAtPt.x = standPos.x, LookAtPt.y = 0.0, LookAtPt.z = standPos.z;
  if (g_width > g_height)
    LookAtPt *= g_height;
  else
    LookAtPt *= g_width;
  w = glm::normalize(-(LookAtPt - camPos));
  u = glm::normalize(glm::cross(Up, w));
  v = glm::cross(w, u); 
  glm::mat4 LookAt = glm::lookAt(camPos, LookAtPt, Up);
  safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(LookAt));
  glm::mat4 rot = glm::inverse(LookAt);
  safe_glUniformMatrix4fv(h_uViewInv, glm::value_ptr(rot));
}

void SetModel(float x, float y, float z, float sx, float sy, float sz, float angleU) {
  glm::mat4 Trans = glm::translate( glm::mat4(1.0f), glm::vec3(x, y, z));
  glm::mat4 Scale = glm::scale( glm::mat4(1.0f), glm::vec3(sx, sy, sz));
  glm::mat4 Rotate = glm::rotate(glm::mat4(1.0f), angleU, axisRot);
  glm::mat3 model;
  
  /*if (transState == 'r')
     rotated = Rotate * rotated;
  angleC = 0.0;*/
  
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Trans * Scale * Rotate));
  model = glm::transpose(glm::inverse(glm::mat3(Trans * Scale * Rotate)));
  glUniformMatrix3fv(h_uModInvTransp, 1, GL_FALSE, (glm::value_ptr(model)));
}

/* set the model transform to the identity */
/*void SetModelT() {
  glm::mat4 tmp = glm::mat4(1.0f);
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(tmp));
  safe_glUniformMatrix4fv(h_uModelMatrix2, glm::value_ptr(tmp));
}*/

/* set the model transform to the identity */
void SetModelI() {
  glm::mat4 tmp = glm::mat4(1.0f);
  glm::mat4 Scale = glm::scale( glm::mat4(1.0f), glm::vec3(10.0, 1.0, 10.0));
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Scale * tmp));
  glUniformMatrix3fv(h_uModInvTransp, 1, GL_FALSE, (glm::value_ptr(glm::mat3(1.0f))));
}

void SetModelS() {
  glm::mat4 tmp = glm::mat4(1.0f);
  glm::mat4 Scale = glm::scale( glm::mat4(1.0f), glm::vec3(30.0, 30.0, 30.0));
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Scale * tmp));
  glUniformMatrix3fv(h_uModInvTransp, 1, GL_FALSE, (glm::value_ptr(glm::mat3(1.0f))));
}

void SetModelL() {
  glm::mat4 Trans = glm::translate(glm::mat4(1.0f), glm::vec3(lightPosition.x, lightPosition.y, lightPosition.z));
  glm::mat4 Scale = glm::scale(glm::mat4(1.0f), glm::vec3(10.0, 10.0, 10.0));
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Trans * Scale));
}

void SetModelO() {
  glm::mat3 model;
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(ModelTrans.modelViewMatrix));
  model = glm::transpose(glm::inverse(glm::mat3(ModelTrans.modelViewMatrix)));
  glUniformMatrix3fv(h_uModInvTransp, 1, GL_FALSE, (glm::value_ptr(model)));
}

/*function to help load the shader  - note current version only loading a vertex shader */
int InstallShader(const GLchar *vShaderName, const GLchar *fShaderName) {
	GLuint VS, FS; //handles to shader object
	GLint vCompiled, fCompiled, linked; //status of shader
	
	VS = glCreateShader(GL_VERTEX_SHADER);
	FS = glCreateShader(GL_FRAGMENT_SHADER);
	
	//load the source
	glShaderSource(VS, 1, &vShaderName, NULL);
	glShaderSource(FS, 1, &fShaderName, NULL);
	
	//compile shader and print log
	glCompileShader(VS);
	/* check shader status requires helper functions */
	printOpenGLError();
	glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
	printShaderInfoLog(VS);

    //compile shader and print log
    glCompileShader(FS);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
    printShaderInfoLog(FS);

    if (!vCompiled || !fCompiled) {	
		printf("Error compiling the shader %s", vShaderName);
		return 0;
	}
	 
	//create a program object and attach the compiled shader
	ShadeProg = glCreateProgram();
	glAttachShader(ShadeProg, VS);
	glAttachShader(ShadeProg, FS);
	
	glLinkProgram(ShadeProg);
	/* check shader status requires helper functions */
	printOpenGLError();
	glGetProgramiv(ShadeProg, GL_LINK_STATUS, &linked);
	printProgramInfoLog(ShadeProg);

	glUseProgram(ShadeProg);
	
	/* get handles to attribute data */
	h_aPosition = safe_glGetAttribLocation(ShadeProg, "aPosition");
	h_aNormal = safe_glGetAttribLocation(ShadeProg,	"aNormal");
	h_aTexCoord = safe_glGetAttribLocation(ShadeProg,  "aTexCoord");
       h_uTexUnit = safe_glGetUniformLocation(ShadeProg, "uTexUnit");
    h_uProjMatrix = safe_glGetUniformLocation(ShadeProg, "uProjMatrix");
    h_uViewMatrix = safe_glGetUniformLocation(ShadeProg, "uViewMatrix");
    h_uModelMatrix = safe_glGetUniformLocation(ShadeProg, "uModelMatrix");
    
    //h_uLightPos = safe_glGetUniformLocation(ShadeProg, "uLPos");
    h_uLightColor = safe_glGetUniformLocation(ShadeProg, "uLCol");
    h_uNasty = safe_glGetUniformLocation(ShadeProg, "uNasty");
    
    h_uMatAmb = safe_glGetUniformLocation(ShadeProg, "uMat.aColor");
    h_uMatDif = safe_glGetUniformLocation(ShadeProg, "uMat.dColor");
    h_uMatSpec = safe_glGetUniformLocation(ShadeProg, "uMat.sColor");
    h_uMatShine = safe_glGetUniformLocation(ShadeProg, "uMat.shine");
    
    h_uViewInv = safe_glGetUniformLocation(ShadeProg, "uViewInv");
    h_uModInvTransp = safe_glGetUniformLocation(ShadeProg, "uModInvTransp");
	printf("sucessfully installed shader %d\n", ShadeProg);
	return 1;
	
}

/* Some OpenGL initialization */
void Initialize ()					// Any GL Init Code 
{
	// Start Of User Initialization
	glClearColor (0.5f, 0.8f, 1.0, 1.0f);								
	// Black Background
 	glClearDepth (1.0f);	// Depth Buffer Setup
 	glDepthFunc (GL_LEQUAL);	// The Type Of Depth Testing
	glEnable (GL_DEPTH_TEST);// Enable Depth Testing
    
    /* texture specific settings */
    glEnable(GL_TEXTURE_2D);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    
	/* some matrix stack init */
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();
}

void box(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[0]->PositionHandle);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[0]->NormalHandle);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, Meshes[0]->IndexHandle);
   glDrawElements(GL_TRIANGLES, Meshes[0]->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
}
void sword(void) {
   ModelTrans.translate(vec3(0.0, -0.35, 0.0));
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[2]->PositionHandle);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[2]->NormalHandle);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, Meshes[2]->IndexHandle);
   glDrawElements(GL_TRIANGLES, Meshes[2]->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
}
/*void mSword(void) {
   SetModelO();
   glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, posSword);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, normSword);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsSword*3);
}*/
void tree(void) {
   glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, posTree);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, normTree);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsTree*3);
}
void head(void) {
   SetModelO();
   glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, posH);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, normH);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsH*3);
}
void neck(void) {   
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posN);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normN);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsN*3);
}
void chest(void) { 
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posC);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normC);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsC*3);
}
void upperArm(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posUA);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normUA);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsUA*3);
}
void lowerArm(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posLA);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normLA);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsLA*3);
}
void upperLeg(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posUL);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normUL);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsUL*3);
}
void lowerLeg(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posLL);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normLL);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsLL*3);
}
void hand(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posHand);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normHand);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsHand*3);
}
void foot(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posFoot);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normFoot);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsFoot*3);
}
void shoulder(void) {
   SetModelO();
   glBindBuffer(GL_ARRAY_BUFFER, posS);
	safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, normS);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glDrawArrays(GL_TRIANGLES, 0, vertsS*3);
}

/* Main display function */
void Draw (void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//Start our shader	
 	glUseProgram(ShadeProg);

    /* only set the projection and view matrix once */
   SetProjectionMatrix();
   SetView();
	
	//SetModelT();
    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, 0);

    safe_glUniform1i(h_uTexUnit, 0);
    
	//data set up to access the vertices and color
  	safe_glEnableVertexAttribArray(h_aPosition);	
   safe_glEnableVertexAttribArray(h_aNormal);	
   //safe_glUniform3f(h_uLightPos, lightPosition.x, lightPosition.y, lightPosition.z);
   safe_glUniform3f(h_uLightColor, lightColor.x, lightColor.y, lightColor.z);

   // MY MODEL PLACING   
   safe_glUniform1i(h_uNasty, 0);
   
   drawMainModel();
	
	SetMaterial(3);
	for (int i = 0; i < numTrees; i++) {
	   SetModel(treeTransX[i], 14.0, treeTransZ[i], 25.0, 30.0, 25.0, 0.0);
   	tree();
   }
	
   //Light/SUN
   safe_glUniform1i(h_uNasty, 1);
   SetModelL();
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[1]->PositionHandle);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ARRAY_BUFFER, Meshes[1]->NormalHandle);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, Meshes[1]->IndexHandle);
   glDrawElements(GL_TRIANGLES, Meshes[1]->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
   
   //PLANE / Ground
   SetModelI();
   glEnable(GL_TEXTURE_2D);
   glActiveTexture(GL_TEXTURE0);
   glBindTexture(GL_TEXTURE_2D, 0);

   safe_glUniform1i(h_uTexUnit, 0);
   safe_glUniform1i(h_uNasty, 2);
   glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ARRAY_BUFFER, GNBuffObj);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
   SetMaterial(1);
   safe_glEnableVertexAttribArray(h_aTexCoord);
   glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj2);
   safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
   glDrawElements(GL_TRIANGLES, g_GiboLen, GL_UNSIGNED_SHORT, 0);
   glDisable(GL_TEXTURE_2D);
   
   glEnable(GL_TEXTURE_2D);
   glActiveTexture(GL_TEXTURE1);
   glBindTexture(GL_TEXTURE_2D, 1);

   safe_glUniform1i(h_uTexUnit, 1);
   safe_glUniform1i(h_uNasty, 2);
   safe_glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);

   safe_glEnableVertexAttribArray(h_aTexCoord);
   glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
   safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 

   // bind ibo
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CIndxBuffObj);
   glDrawElements(GL_TRIANGLES, g_CiboLen, GL_UNSIGNED_SHORT, 0);
    
	//clean up 
	safe_glDisableVertexAttribArray(h_aPosition);
	safe_glDisableVertexAttribArray(h_aNormal);
	//disable the shader
	glUseProgram(0);
	glDisable(GL_TEXTURE_2D);
	glutSwapBuffers();
}

/* Reshape */
void ReshapeGL (int width, int height)
{
     g_width = (float)width;
     g_height = (float)height;
     glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));
}

void mouse(int button, int state, int x, int y) {
   if (button == GLUT_LEFT_BUTTON) {
      if (state == GLUT_DOWN) {
     	   if (g_width > g_height) {
	         NewPoint[0] = x;// * (g_width / g_height);
	         NewPoint[1] = y;
	      }
	      else {
	         NewPoint[0] = x;
	         NewPoint[1] = y;// / (g_width / g_height);
	      }
      }
   }
}

//the keyboard callback to change the values to the transforms
void keyboard(unsigned char key, int x, int y ){
  switch( key ) {
    /* WASD keyes effect view/camera transform */
    case 'i':
      if (camState == 'n' && iDown == false)
         camState = 'f';
      iDown = true;
      break;
    case 'k':
      if (camState == 'n' && kDown == false)
         camState = 'b';
      kDown = true;
      break;
    case 'j':
      if (camState == 'n' && jDown == false)
         camState = 'l';
      jDown = true;
      break;
    case 'l':
      if (camState == 'n' && lDown == false)
         camState = 'r';
      lDown = true;
      break;
    case 'q': case 'Q' :
      exit( EXIT_SUCCESS );
      break;
    case 'r':
      backState = 'r';
      break;
    case 't':
      backState = 'w';
      break;
    /*case 'y':
      mainState = 's';
      curARU = {fromSARU.x, fromSARU.y, fromSARU.z};
      curALU = {fromSALU.x, fromSALU.y, fromSALU.z};
      curARL = {fromSARL.x, fromSARL.y, fromSARL.z};
      curALL = {fromSALL.x, fromSALL.y, fromSALL.z};
      curLRU = {fromSLRU.x, fromSLRU.y, fromSLRU.z};
      curLLU = {fromSLLU.x, fromSLLU.y, fromSLLU.z};
      curLRL = {fromSLRL.x, fromSLRL.y, fromSLRL.z};
      curLLL = {fromSLLL.x, fromSLLL.y, fromSLLL.z};
      curT = {fromST.x, fromST.y, fromST.z};
      curH = {fromSH.x, fromSH.y, fromSH.z};
      curN = {fromSN.x, fromSN.y, fromSN.z};
      curRH = {fromSRH.x, fromSRH.y, fromSRH.z};
      curLH = {fromSLH.x, fromSLH.y, fromSLH.z};
      curRF = {fromSRF.x, fromSRF.y, fromSRF.z};
      curLF = {fromSLF.x, fromSLF.y, fromSLF.z};
      break;*/
    case 'w':
      if (moveCase == 'n' && wDown == false) {
         mainState = backState;
         if (mainState == 'r') {
            curARU = {fromRARU.x, fromRARU.y, fromRARU.z};
            curALU = {fromRALU.x, fromRALU.y, fromRALU.z};
            curARL = {fromRARL.x, fromRARL.y, fromRARL.z};
            curALL = {fromRALL.x, fromRALL.y, fromRALL.z};
            curLRU = {fromRLRU.x, fromRLRU.y, fromRLRU.z};
            curLLU = {fromRLLU.x, fromRLLU.y, fromRLLU.z};
            curLRL = {fromRLRL.x, fromRLRL.y, fromRLRL.z};
            curLLL = {fromRLLL.x, fromRLLL.y, fromRLLL.z};
            curT = {fromRT.x, curT.y, fromRT.z};
            curH = {fromRH.x, fromRH.y, fromRH.z};
            curN = {fromRN.x, fromRN.y, fromRN.z};
            curRH = {fromRRH.x, fromRRH.y, fromRRH.z};
            curLH = {fromRLH.x, fromRLH.y, fromRLH.z};
            curRF = {fromRRF.x, fromRRF.y, fromRRF.z};
            curLF = {fromRLF.x, fromRLF.y, fromRLF.z};
         }
         else if (mainState == 'w') {
            curARU = {fromWARU.x, fromWARU.y, fromWARU.z};
            curALU = {fromWALU.x, fromWALU.y, fromWALU.z};
            curARL = {fromWARL.x, fromWARL.y, fromWARL.z};
            curALL = {fromWALL.x, fromWALL.y, fromWALL.z};
            curLRU = {fromWLRU.x, fromWLRU.y, fromWLRU.z};
            curLLU = {fromWLLU.x, fromWLLU.y, fromWLLU.z};
            curLRL = {fromWLRL.x, fromWLRL.y, fromWLRL.z};
            curLLL = {fromWLLL.x, fromWLLL.y, fromWLLL.z};
            curT = {fromWT.x, curT.y, fromWT.z};
            curH = {fromWH.x, fromWH.y, fromWH.z};
            curN = {fromWN.x, fromWN.y, fromWN.z};
            curRH = {fromWRH.x, fromWRH.y, fromWRH.z};
            curLH = {fromWLH.x, fromWLH.y, fromWLH.z};
            curRF = {fromWRF.x, fromWRF.y, fromWRF.z};
            curLF = {fromWLF.x, fromWLF.y, fromWLF.z};
         }
         moveCase = 'f';
      }
      wDown = true;
      break;
    case 'a':
      if (turnCase == 'n' && aDown == false)
         turnCase = 'l';
      aDown = true;
      break;
    case 's':
      if (moveCase == 'n' && sDown == false)
         moveCase = mainState = 'b';
      sDown = true;
      break;
    case 'd':
      if (turnCase == 'n' && dDown == false)
         turnCase = 'r';
      dDown = true;
      break;
  }
  glutPostRedisplay();
}

void keyboardUp(unsigned char key, int x, int y)
{
   switch (key) {
      case 'w':
         wDown = false;
         mainState = 's';
         moveCase = 'n';
         break;
      case 'a':
         aDown = false;
         //mainState = 's';
         turnCase = 'n';
         break;
      case 's':
         sDown = false;
         mainState = 's';
         moveCase = 'n';
         break;
      case 'd':
         dDown = false;
         //mainState = 's';
         turnCase = 'n';
         break;
      case 'j':
         jDown = false;
         camState = 'n';
         break;
      case 'l':
         lDown = false;
         camState = 'n';
         break;
      case 'i':
         iDown = false;
         camState = 'n';
         break;
      case 'k':
         kDown = false;
         camState = 'n';
         break;
   }
   if (key == 'w' || key == 's') {
      curARU = {fromSARU.x, fromSARU.y, fromSARU.z};
      curALU = {fromSALU.x, fromSALU.y, fromSALU.z};
      curARL = {fromSARL.x, fromSARL.y, fromSARL.z};
      curALL = {fromSALL.x, fromSALL.y, fromSALL.z};
      curLRU = {fromSLRU.x, fromSLRU.y, fromSLRU.z};
      curLLU = {fromSLLU.x, fromSLLU.y, fromSLLU.z};
      curLRL = {fromSLRL.x, fromSLRL.y, fromSLRL.z};
      curLLL = {fromSLLL.x, fromSLLL.y, fromSLLL.z};
      curT = {fromST.x, curT.y, fromST.z};
      curH = {fromSH.x, fromSH.y, fromSH.z};
      curN = {fromSN.x, fromSN.y, fromSN.z};
      curRH = {fromSRH.x, fromSRH.y, fromSRH.z};
      curLH = {fromSLH.x, fromSLH.y, fromSLH.z};
      curRF = {fromSRF.x, fromSRF.y, fromSRF.z};
      curLF = {fromSLF.x, fromSLF.y, fromSLF.z};
   }
}
void camChange(int x, int y) {
   float temp = alpha + ((float)(y - NewPoint[1]) * PI) / g_height;
   
   if ((temp < (80.0 * PI / 180.0) || temp < 0.0) && (temp > (-80.0 * PI / 180.0) || temp > 0.0))
      alpha += (((float)(y - NewPoint[1]) * PI) / g_height);
   
   beta += ((float)(x - NewPoint[0]) * PI) / g_width;
   NewPoint[0] = (float)x;
   NewPoint[1] = (float)y;
   
   glutPostRedisplay();
}

int main( int argc, char *argv[] )
{
   lightPosition = glm::vec3(0.0, 70.0, -40.0);
   lightColor = glm::vec3(1.0, 1.0, 1.0);
   
   srand (time(NULL));
   
   for (int i = 0; i < numTrees; i++) {
      treeTransX[i] = -40.0 + (float)(rand() % 80);
      treeTransZ[i] = -40.0 + (float)(rand() % 80);
      for (int j = 0; j < numTrees; j++) {
         if (j != i)
            if (sqrt(((treeTransX[i] - treeTransX[j]) * (treeTransX[i] - treeTransX[j])) +
                     ((treeTransZ[i] - treeTransZ[j]) * (treeTransZ[i] - treeTransZ[j]))) < 4.0) {
               i--;
               j = numTrees;
            }
      }
   }
   
   glutInit( &argc, argv );
   glutInitWindowPosition( 200, 200 );
   glutInitWindowSize( 400, 400 );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   glutCreateWindow("My first mesh");
   glutReshapeFunc( ReshapeGL );
   glutDisplayFunc( Draw );
   glutKeyboardFunc( keyboard );
   glutIgnoreKeyRepeat	( 1 );
   glutKeyboardUpFunc( keyboardUp );
   glutMouseFunc( mouse );
   glutMotionFunc( camChange );
   glutIdleFunc( animate );
		
	g_width = g_height = 200;

#ifdef _WIN32 
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		std::cerr << "Error initializing glew! " << glewGetErrorString(err) << std::endl;
		return 1;
	}
#endif

   Initialize();
	LoadTexture((char *)"gras.bmp", 0);
	LoadTexture((char *)"forest.bmp", 1);
	//test the openGL version
	getGLversion();
	//install the shader
	if (!InstallShader(textFileRead((char *)"mesh_vert.glsl"), textFileRead((char *)"mesh_frag.glsl")))   {
		printf("Error installing shader!\n");
		return 0;
	}
		
	InitGeom();
  	glutMainLoop();
   return 0;
}

//routines to load in a bmp files - must be 2^nx2^m and a 24bit bmp
GLvoid LoadTexture(char* image_file, int texID) { 
  
  TextureImage = (Image *) malloc(sizeof(Image));
  if (TextureImage == NULL) {
    printf("Error allocating space for image");
    exit(1);
  }
  cout << "trying to load " << image_file << endl;
  if (!ImageLoad(image_file, TextureImage)) {
    exit(1);
  }  
  /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
  /*  x size from image, y size from image,                                              */    
  /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
  glBindTexture(GL_TEXTURE_2D, texID);
  glTexImage2D(GL_TEXTURE_2D, 0, 3,
    TextureImage->sizeX, TextureImage->sizeY,
    0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
  
}

/* BMP file loader loads a 24-bit bmp file only */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
static unsigned int getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0; 
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}

void drawMainModel(void) {
   SetMaterial(0);
   //TORSO
   ModelTrans.pushMatrix();
      ModelTrans.translate(vec3(standPos.x, standPos.y / 4.0, standPos.z - 5.0));
      ModelTrans.scale(.25, .25, .25);
      ModelTrans.rotate(180.0, vec3(0, 1, 0));
      ModelTrans.rotate(curT.y, vec3(0, 1, 0));
      ModelTrans.rotate(curT.x, vec3(1, 0, 0));
      ModelTrans.rotate(curT.z, vec3(0, 0, 1));
      ModelTrans.scale(BSx, BSy, mainZ);
      ModelTrans.scale(BSOx, BSOy, BSOz);
      //NECK
      ModelTrans.pushMatrix();
         ModelTrans.scale(1.0 / BSx, 1.0 / BSy, 1.0 / mainZ);
         ModelTrans.scale(1.0 / BSOx, 1.0 / BSOy, 1.0 / BSOz);
         ModelTrans.translate(vec3(0.0, (BSy / 2.0) + (0.125 / 2.0), 0.0));
         ModelTrans.rotate(curN.x, vec3(1, 0, 0));
         ModelTrans.rotate(curN.y, vec3(0, 1, 0));
         ModelTrans.rotate(curN.z, vec3(0, 0, 1));
         ModelTrans.translate(vec3(0.0, 0.125 / 2.0, 0.0));
         ModelTrans.scale(1.0 * 0.95, 0.75, 0.75 * 0.95);
         ModelTrans.scale(NSOx, NSOy, NSOz);
         ModelTrans.translate(vec3(NTOx, NTOy, NTOz));
         //HEAD
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-NTOx, -NTOy, -NTOz));
            ModelTrans.scale(1.0 / 0.95, 1.0 / 0.75, 1.0 / 0.75 * 0.95);
            ModelTrans.scale(1.0 / NSOx, 1.0 / NSOy, 1.0 / NSOz);
            ModelTrans.translate(vec3(0.0, 0.0, 0.1875));
            ModelTrans.rotate(curH.x, vec3(1, 0, 0));
            ModelTrans.rotate(curH.y, vec3(0, 1, 0));
            ModelTrans.rotate(curH.z, vec3(0, 0, 1));
            ModelTrans.translate(vec3(0.0, 1.125, 0.1875));
            ModelTrans.translate(vec3(0.0, 1.125, 0.0));
            ModelTrans.scale(2.0, 2.5, mainZ);
            ModelTrans.scale(HeSOx, HeSOy, HeSOz);
            ModelTrans.translate(vec3(HeTOx, HeTOy, HeTOz));
            SetMaterial(0);
            head();
         ModelTrans.popMatrix();
         SetMaterial(0);
         neck();
      ModelTrans.popMatrix();
      //LEFT SHOULDER
      SetMaterial(0);
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(0.175, (BSy / 2.0) - (ASx / 2.0) - 0.5, -0.025));
         ModelTrans.translate(vec3(0.0, -(ASy/2.0) + (ASx / 2.0), 0.0));
         ModelTrans.scale(1.2, 0.5, 1.0);
         shoulder();
      ModelTrans.popMatrix();
      //RIGHT SHOULDER
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(-0.175, (BSy / 2.0) - (ASx / 2.0) - 0.5, -0.025));
         ModelTrans.translate(vec3(0.0, -(ASy/2.0) + (ASx / 2.0), 0.0));
         ModelTrans.scale(-1.2, 0.5, 1.0);
         shoulder();
      ModelTrans.popMatrix();
      //UPPER RIGHT ARM
      ModelTrans.pushMatrix();
         ModelTrans.scale(1.0 / BSx, 1.0 / BSy, 1.0 / mainZ);
         ModelTrans.scale(1.0 / BSOx, 1.0 / BSOy, 1.0 / BSOz);
         ModelTrans.translate(vec3((-BSx / 2.0) - (ASx / 2.0), (BSy / 2.0) - (ASx / 2.0) , 0.0));
         ModelTrans.rotate(curARU.x, vec3(1, 0, 0));
         ModelTrans.rotate(curARU.y, vec3(0, 1, 0));
         ModelTrans.rotate(curARU.z, vec3(0, 0, 1));
         ModelTrans.translate(vec3(0.0, -(ASy/2.0) + (ASx / 2.0), 0.0));
         ModelTrans.scale(ASx * 0.95, ASy, mainZ * 0.95);
         ModelTrans.scale(ASOx, ASOy, ASOz);
         ModelTrans.translate(vec3(ATOx, ATOy, ATOz));
         //LOWER RIGHT ARM
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-ATOx, -ATOy, -ATOz));
            ModelTrans.scale(1.0 / ASx * 0.95, 1.0 / ASy, 1.0 / mainZ * 0.95);
            ModelTrans.scale(1.0 / ASOx, 1.0 / ASOy, 1.0 / ASOz);
            ModelTrans.translate(vec3(0.0, -(ASy / 2.0) + 0.35, 0.0));
            ModelTrans.rotate(curARL.x, vec3(1, 0, 0));
            ModelTrans.rotate(curARL.y, vec3(0, 1, 0));
            ModelTrans.rotate(curARL.z, vec3(0, 0, 1));
            ModelTrans.translate(vec3(0.0, -(ASy/2.0) + (ASx / 2.0), 0.0));
            ModelTrans.scale(ASx, ASy, mainZ);
            ModelTrans.scale(ASOLx, ASOLy, ASOLz);
            ModelTrans.translate(vec3(ATOLx, ATOLy, ATOLz));
            //RIGHT HAND
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(-ATOLx, -ATOLy, -ATOLz));
               ModelTrans.scale(1.0 / ASx, 1.0 / ASy, 1.0 / mainZ);
               ModelTrans.scale(1.0 / ASOLx, 1.0 / ASOLy, 1.0 / ASOLz);
               ModelTrans.translate(vec3(0.0, -(ASy / 2.0) + 0.60, -0.30));
               ModelTrans.rotate(curRH.x, vec3(1, 0, 0));
               ModelTrans.rotate(curRH.y, vec3(0, 1, 0));
               ModelTrans.rotate(curRH.z, vec3(0, 0, 1));
               ModelTrans.translate(vec3(0.0, -0.75 + 0.25, 0.0));
               ModelTrans.scale(ASx * 0.95, 1.5, mainZ * 0.95);
               ModelTrans.scale(HSOx, HSOy, HSOz);
               ModelTrans.translate(vec3(HTOx, HTOy, HTOz));
               SetMaterial(1);
               hand();
            ModelTrans.popMatrix();
            SetMaterial(1);
            lowerArm();
         ModelTrans.popMatrix();
         SetMaterial(0);
         upperArm();
      ModelTrans.popMatrix();
      //UPPER LEFT ARM
      ModelTrans.pushMatrix();
         ModelTrans.scale(1.0 / BSx, 1.0 / BSy, 1.0 / mainZ);
         ModelTrans.scale(1.0 / BSOx, 1.0 / BSOy, 1.0 / BSOz);
         ModelTrans.translate(vec3((BSx / 2.0) + (ASx / 2.0), (BSy / 2.0) - (ASx / 2.0) , 0.0));
         ModelTrans.rotate(curALU.x, vec3(1, 0, 0));
         ModelTrans.rotate(curALU.y, vec3(0, 1, 0));
         ModelTrans.rotate(curALU.z, vec3(0, 0, 1));
         ModelTrans.translate(vec3(0.0, -(ASy/2.0) + (ASx / 2.0), 0.0));
         ModelTrans.scale(ASx * 0.95, ASy, mainZ * 0.95);
         ModelTrans.scale(ASOx, ASOy, ASOz);
         ModelTrans.translate(vec3(ATOx, ATOy, ATOz));
         //LOWER LEFT ARM
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-ATOx, -ATOy, -ATOz));
            ModelTrans.scale(1.0 / ASx * 0.95, 1.0 / ASy, 1.0 / mainZ * 0.95);
            ModelTrans.scale(1.0 / ASOx, 1.0 / ASOy, 1.0 / ASOz);
            ModelTrans.translate(vec3(0.0, -(ASy / 2.0) + 0.35, 0.0));
            ModelTrans.rotate(curALL.x, vec3(1, 0, 0));
            ModelTrans.rotate(curALL.y, vec3(0, 1, 0));
            ModelTrans.rotate(curALL.z, vec3(0, 0, 1));
            ModelTrans.translate(vec3(0.0, -(ASy / 2.0) + (ASx / 2.0), 0.0));
            ModelTrans.scale(ASx, ASy, mainZ);
            ModelTrans.scale(ASOLx, ASOLy, ASOLz);
            ModelTrans.translate(vec3(ATOLx, ATOLy, ATOLz));
            //LEFT HAND
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(-ATOLx, -ATOLy, -ATOLz));
               ModelTrans.scale(1.0 / ASx, 1.0 / ASy, 1.0 / mainZ);
               ModelTrans.scale(1.0 / ASOLx, 1.0 / ASOLy, 1.0 / ASOLz);
               ModelTrans.translate(vec3(0.0, -(ASy / 2.0) + 0.60, -0.30));
               ModelTrans.rotate(curLH.x, vec3(1, 0, 0));
               ModelTrans.rotate(curLH.y, vec3(0, 1, 0));
               ModelTrans.rotate(curLH.z, vec3(0, 0, 1));
               ModelTrans.translate(vec3(0.0, -0.75 + 0.25, 0.0));
               ModelTrans.scale(ASx * 0.95, 1.5, mainZ * 0.95);
               ModelTrans.scale(HSOx, HSOy, HSOz);
               ModelTrans.translate(vec3(HTOx, HTOy, HTOz));
               ModelTrans.pushMatrix();
                  ModelTrans.translate(vec3(-HTOx, -HTOy, -HTOz));
                  ModelTrans.scale(1.0 / HSOx, 1.0 / HSOy, 1.0 / HSOz);
                  SetMaterial(4);
                  sword();
                  //mSword();
               ModelTrans.popMatrix();
               SetMaterial(1);
               hand();
            ModelTrans.popMatrix();
            SetMaterial(1);
            lowerArm();
         ModelTrans.popMatrix();
         SetMaterial(0);
         upperArm();
      ModelTrans.popMatrix();
      //UPPER RIGHT LEG
      ModelTrans.pushMatrix();
         ModelTrans.scale(1.0 / BSx, 1.0 / BSy, 1.0 / mainZ);
         ModelTrans.scale(1.0 / BSOx, 1.0 / BSOy, 1.0 / BSOz);
         ModelTrans.translate(vec3(-(BSx / 2.0) + (LSx / 2.0), -(BSy / 2.0), 0.0));
         ModelTrans.rotate(curLRU.x, vec3(1, 0, 0));
         ModelTrans.rotate(curLRU.y, vec3(0, 1, 0));
         ModelTrans.rotate(curLRU.z, vec3(0, 0, 1));
         ModelTrans.translate(vec3(0.0, -(LSy/2.0) + (LSx / 2.0), 0.0));
         ModelTrans.scale(-LSx * 0.95, LSy, mainZ * 0.95);
         ModelTrans.scale(LSOx, LSOy, LSOz);
         ModelTrans.translate(vec3(LTOx, LTOy, LTOz));
         //LOWER RIGHT LEG
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-LTOx, -LTOy, -LTOz));
            ModelTrans.scale(1.0 / LSx * 0.95, 1.0 / LSy, 1.0 / mainZ * 0.95);
            ModelTrans.scale(1.0 / LSOx, 1.0 / LSOy, 1.0 / LSOz);
            ModelTrans.translate(vec3(0.0, -LSy / 2.0 + 0.25, 0.0));
            ModelTrans.rotate(curLRL.x, vec3(1, 0, 0));
            ModelTrans.rotate(curLRL.y, vec3(0, 1, 0));
            ModelTrans.rotate(-curLRL.z, vec3(0, 0, 1));
            ModelTrans.translate(vec3(0.0, -(LSy/2.0) + (LSx / 2.0), 0.0));
            ModelTrans.scale(LSx, LSy, mainZ);
            ModelTrans.scale(LSOLx, LSOLy, LSOLz);
            ModelTrans.translate(vec3(LTOLx, LTOLy, LTOLz));
            //RIGHT FOOT
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(-LTOLx, -LTOLy, -LTOLz));
               ModelTrans.scale(1.0 / LSx, 1.0 / LSy, 1.0 / mainZ);
               ModelTrans.scale(1.0 / LSOLx, 1.0 / LSOLy, 1.0 / LSOLz);
               ModelTrans.translate(vec3(0.0, -(LSy / 2.0) - 0.5 + 1.85, 0.25));
               ModelTrans.rotate(curRF.x, vec3(1, 0, 0));
               ModelTrans.rotate(curRF.y, vec3(0, 1, 0));
               ModelTrans.rotate(-curRF.z, vec3(0, 0, 1));
               ModelTrans.translate(vec3(0.0, 0.0, LSx / 2.0));
               ModelTrans.scale(LSx * 0.95, LSx, mainZ * 2.0 * .95);
               ModelTrans.scale(FSOx, FSOy, FSOz);
               ModelTrans.translate(vec3(FTOx, FTOy, FTOz));
               SetMaterial(1);
               foot();
            ModelTrans.popMatrix();
            SetMaterial(1);
            lowerLeg();
         ModelTrans.popMatrix();
         SetMaterial(5);
         upperLeg();
      ModelTrans.popMatrix();
      //UPPER LEFT LEG
      ModelTrans.pushMatrix();
         ModelTrans.scale(1.0 / BSx, 1.0 / BSy, 1.0 / mainZ);
         ModelTrans.scale(1.0 / BSOx, 1.0 / BSOy, 1.0 / BSOz);
         ModelTrans.translate(vec3((BSx / 2.0) - (LSx / 2.0), -(BSy / 2.0), 0.0));
         ModelTrans.rotate(curLLU.x, vec3(1, 0, 0));
         ModelTrans.rotate(curLLU.y, vec3(0, 1, 0));
         ModelTrans.rotate(curLLU.z, vec3(0, 0, 1));
         ModelTrans.translate(vec3(0.0, -(LSy/2.0) + (LSx / 2.0), 0.0));
         ModelTrans.scale(LSx * 0.95, LSy, mainZ * 0.95);
         ModelTrans.scale(LSOx, LSOy, LSOz);
         ModelTrans.translate(vec3(LTOx, LTOy, LTOz));
         //LOWER LEFT LEG
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-LTOx, -LTOy, -LTOz));
            ModelTrans.scale(1.0 / LSx * 0.95, 1.0 / LSy, 1.0 / mainZ * 0.95);
            ModelTrans.scale(1.0 / LSOx, 1.0 / LSOy, 1.0 / LSOz);
            ModelTrans.translate(vec3(0.0, -LSy / 2.0 + 0.25, 0.0));
            ModelTrans.rotate(curLLL.x, vec3(1, 0, 0));
            ModelTrans.rotate(curLLL.y, vec3(0, 1, 0));
            ModelTrans.rotate(curLLL.z, vec3(0, 0, 1));
            ModelTrans.translate(vec3(0.0, -(LSy / 2.0) + (LSx / 2.0), 0.0));
            ModelTrans.scale(LSx, LSy, mainZ);
            ModelTrans.scale(LSOLx, LSOLy, LSOLz);
            ModelTrans.translate(vec3(LTOLx, LTOLy, LTOLz));
            //LEFT FOOT
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(-LTOLx, -LTOLy, -LTOLz));
               ModelTrans.scale(1.0 / LSx, 1.0 / LSy, 1.0 / mainZ);
               ModelTrans.scale(1.0 / LSOLx, 1.0 / LSOLy, 1.0 / LSOLz);
               ModelTrans.translate(vec3(0.0, -(LSy / 2.0) - 0.5 + 1.85, 0.25));
               ModelTrans.rotate(curLF.x, vec3(1, 0, 0));
               ModelTrans.rotate(curLF.y, vec3(0, 1, 0));
               ModelTrans.rotate(curLF.z, vec3(0, 0, 1));
               ModelTrans.translate(vec3(0.0, 0.0, LSx / 2.0));
               ModelTrans.scale(LSx * 0.95, LSx, mainZ * 2.0 * .95);
               ModelTrans.scale(FSOx, FSOy, FSOz);
               ModelTrans.translate(vec3(FTOx, FTOy, FTOz));
               SetMaterial(1);
               foot();
            ModelTrans.popMatrix();
            SetMaterial(1);
            lowerLeg();
         ModelTrans.popMatrix();
         SetMaterial(5);
         upperLeg();
      ModelTrans.popMatrix();
      SetMaterial(0);
      chest();
   ModelTrans.popMatrix();
}
void animate(void) {
   float period = 0.0;
   //DECIDE LATER WHAT KIND OF STANDING I WANT
   if (mainState == 's') {
      period = 100.0;
      if (standUp) {
         if (curLH.y >= fromSLH.y)
            standUp = false;
         else {
            /*curLLU.x += (fromSLLU.x - toSLLU.x) / period;
            curLLL.x += (fromSLLL.x - toSLLL.x) / period;
            curLRU.x += (fromSLRU.x - toSLRU.x) / period;
            curLRL.x += (fromSLRL.x - toSLRL.x) / period;
            curLF.x += (fromSLF.x - toSLF.x) / period;
            curRF.x += (fromSRF.x - toSRF.x) / period;
            curLH.y += (fromSLH.y - toSLH.y) / period;
            standPos.y =  origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            standPos.z =  origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));*/
            curLH.y += (fromSLH.y - toSLH.y) / period;
            curLH.z += (fromSLH.z - toSLH.z) / period;
         }
      }
      else {
         if (curLH.y <= toSLH.y)
            standUp = true;
         else {
            /*curLLU.x += (toSLLU.x - fromSLLU.x) / period;
            curLLL.x += (toSLLL.x - fromSLLL.x) / period;
            curLRU.x += (toSLRU.x - fromSLRU.x) / period;
            curLRL.x += (toSLRL.x - fromSLRL.x) / period;
            curLF.x += (toSLF.x - fromSLF.x) / period;
            curRF.x += (toSRF.x - fromSRF.x) / period;
            curLH.y += (toSLH.y - fromSLH.y) / period;
            standPos.y = origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            standPos.z = origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));*/
            curLH.y += (toSLH.y - fromSLH.y) / period;
            curLH.z += (toSLH.z - fromSLH.z) / period;
         }
      }
   }
   else if (mainState == 'w' && wDown) {
      period = 30.0;
      if (leftArmForward) {
         if (curALU.x <= fromWALU.x)
            leftArmForward = false;
         else {
            curALU.x += (fromWALU.x - toWALU.x) / period;
            curARU.x += (fromWARU.x - toWARU.x) / period;
            curLH.y += (fromWLH.y - toWLH.y) / period;
            curLLU.x += (fromWLLU.x - toWLLU.x) / period;
            curLLL.x += (fromWLLL.x - toWLLL.x) / period;
            curLRU.x += (fromWLRU.x - toWLRU.x) / period;
            if (curLRU.x >= 0) {
               curLRL.x -= (fromWLRU.x - toWLRU.x) / (period / 2.0);
               midWRx = curLRL.x;
            }
            else {
               curLRL.x += (fromWLRL.x - midWRx) / (period / 2.0);
               curRF.x += (fromWRF.x - toWRF.x) / (period / 2.0);
            }
            curLF.x += (fromWLF.x - toWLF.x) / period;
            
            //standPos.y =  origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            //standPos.z =  origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));
         }
      }
      else {
         if (curALU.x >= toWALU.x)
            leftArmForward = true;
         else {
            curALU.x += (toWALU.x - fromWALU.x) / period;
            curARU.x += (toWARU.x - fromWARU.x) / period;
            curLH.y += (toWLH.y - fromWLH.y) / period;
            curLLU.x += (toWLLU.x - fromWLLU.x) / period;
            if (curLLU.x >= 0) {
               curLLL.x -= (toWLLU.x - fromWLLU.x) / (period / 2.0);
               midWLx = curLLL.x;
            }
            else {
               curLLL.x += (toWLLL.x - midWLx) / (period / 2.0);
               curLF.x += (toWLF.x - fromWLF.x) / (period / 2.0);
            }
            curLRU.x += (toWLRU.x - fromWLRU.x) / period;
            curLRL.x += (toWLRL.x - fromWLRL.x) / period;
            curRF.x += (toWRF.x - fromWRF.x) / period;
            //standPos.y = origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            //standPos.z = origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));
         }
      }
   }
   else if (mainState == 'b' && sDown) {
      period = 30.0;
      if (leftArmForward) {
         if (curLRU.x >= 5.0)
            leftArmForward = false;
         else {
            curLLU.x += (-10.0) / period;
            curLRU.x -= (-10.0) / period;            
         }
      }
      else {
         if (curLRU.x <= -10.0)
            leftArmForward = true;
         else {
            curLLU.x += (10.0) / period;
            curLRU.x -= (10.0) / period;
            //standPos.y = origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            //standPos.z = origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));
         }
      }
   }
   else if (mainState == 'r' && wDown) {
      period = 15.0;
      if (!rightArmForward) {
         if (curARU.x >= fromRARU.x)
            rightArmForward = true;
         else {
            curALU.x += (fromRALU.x - toRALU.x) / period;
            curARU.x += (fromRARU.x - toRARU.x) / period;
            
            curLLU.x += (fromRLLU.x - toRLLU.x) / period;
            if (curLLU.x <= -fromRT.x) {
               curLLL.x += (fromRLLL.x - toRLLL.x) / (period * 2.0 / 3.0);
               curLF.x += (fromRLF.x - toRLF.x) / (period * 2.0 / 3.0);
            }
            else
               if (curLF.x > fromRLF.x)
                  curLF.x = -curLLU.x - curLLL.x - curT.x;
               else
                  curLF.x = fromRLF.x;
               
            curLRU.x += (fromRLRU.x - toRLRU.x) / period;
            if (curLRU.x >= -fromRT.x)
               curLRL.x += (midRLRL.x - toRLRL.x) / (period / 3.0);
            else {
               curLRL.x += (fromRLRL.x - midRLRL.x) / (period * 2.0 / 3.0);
            }
               
            curLH.y += (fromRLH.y - toRLH.y) / period;
            curLH.x += (fromRLH.x - toRLH.x) / period;
            //standPos.y =  origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            //standPos.z =  origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));
         }
      }
      else {
         if (curARU.x <= toRARU.x)
            rightArmForward = false;
         else {
            curALU.x += (toRALU.x - fromRALU.x) / period;
            curARU.x += (toRARU.x - fromRARU.x) / period;
            
            curLLU.x += (toRLLU.x - fromRLLU.x) / period;
            if (curLLU.x >= -fromRT.x)
               curLLL.x += (midRLLL.x - fromRLLL.x) / (period / 3.0);
            else {
               curLLL.x += (toRLLL.x - midRLLL.x) / (period * 2.0 / 3.0);
            }
            curLRU.x += (toRLRU.x - fromRLRU.x) / period;
            if (curLRU.x <= -fromRT.x) {
               curLRL.x += (toRLRL.x - fromRLRL.x) / (period * 2.0 / 3.0);
               curRF.x += (toRRF.x - fromRRF.x) / (period * 2.0 / 3.0);
            }
            else
               if (curRF.x > toRRF.x)
                  curRF.x = -curLRU.x - curLRL.x - curT.x;
               else
                  curRF.x = toRRF.x;
               
            curLH.y += (toRLH.y - fromRLH.y) / period;
            curLH.x += (toRLH.x - fromRLH.x) / period;
            //standPos.y = origPos.y - 2.0 * (4.0 - ((cos(abs(curLLU.x) * PI / 180.0)) * 4.0));
            //standPos.z = origPos.z - (4.0 - ((sin(abs(curLLU.x) * PI / 180.0)) * 4.0));
         }
      }
   }
   if (turnCase != 'n') {
      if (turnCase == 'l' && aDown) {
         if (mainState == 's') {
            posAng += 1.5;
         }
         else {
            if (mainState == 'r')
               posAng += 0.75;
            else
               posAng += 1.0;
         }
      }
      else if (turnCase == 'r' && dDown) {
         if (mainState == 's') {
            posAng -= 1.5;
         }
         else {
            if (mainState == 'r')
               posAng -= 0.75;
            else
               posAng -= 1.0;
         }
      }
   }
   if (moveCase != 'n') {
      if (moveCase == 'f' && wDown) {
         if (mainState == 'r') {
            if ((posAng <= 0.0 && posAng >= -180.0) || (posAng >= 180.0 && posAng <= 360.0)) {
               standPos.x += .25 * abs(sin(posAng * PI / 180.0));
               camPos.x += .25 * abs(sin(posAng * PI / 180.0));
            }
            else {
               standPos.x += .25 * -abs(sin(posAng * PI / 180.0));
               camPos.x += .25 * -abs(sin(posAng * PI / 180.0));
            }
            if ((posAng <= 90.0 && posAng >= -90.0) || posAng <= -270.0 || posAng >= 270.0) {
               standPos.z -= .25 * abs(cos(posAng * PI / 180.0));
               camPos.z -= .25 * abs(cos(posAng * PI / 180.0));
            }
            else {
               standPos.z -= .25 * -abs(cos(posAng * PI / 180.0));
               camPos.z -= .25 * -abs(cos(posAng * PI / 180.0));
            }
         }
         else if (mainState == 'w') {
            if ((posAng <= 0.0 && posAng >= -180.0) || (posAng >= 180.0 && posAng <= 360.0)) {
               standPos.x += .05 * abs(sin(posAng * PI / 180.0));
               camPos.x += .05 * abs(sin(posAng * PI / 180.0));
            }
            else {
               standPos.x += .05 * -abs(sin(posAng * PI / 180.0));
               camPos.x += .05 * -abs(sin(posAng * PI / 180.0));
            }
            if ((posAng <= 90.0 && posAng >= -90.0) || posAng <= -270.0 || posAng >= 270.0) {
               standPos.z -= .05 * abs(cos(posAng * PI / 180.0));
               camPos.z -= .05 * abs(cos(posAng * PI / 180.0));
            }
            else {
               standPos.z -= .05 * -abs(cos(posAng * PI / 180.0));
               camPos.z -= .05 * -abs(cos(posAng * PI / 180.0));
            }
         }
      }
      else if (moveCase == 'b' && sDown) {
         if ((posAng <= 0.0 && posAng >= -180.0) || (posAng >= 180.0 && posAng <= 360.0)) {
            standPos.x -= .025 * abs(sin(posAng * PI / 180.0));
            camPos.x -= .025 * abs(sin(posAng * PI / 180.0));
         }
         else {   
            standPos.x -= .025 * -abs(sin(posAng * PI / 180.0));
            camPos.x -= .025 * -abs(sin(posAng * PI / 180.0));
         }
         if ((posAng <= 90.0 && posAng >= -90.0) || posAng <= -270.0 || posAng >= 270.0) {
            standPos.z += .025 * abs(cos(posAng * PI / 180.0));
            camPos.z += .025 * abs(cos(posAng * PI / 180.0));
         }
         else {
            standPos.z += .025 * -abs(cos(posAng * PI / 180.0));
            camPos.z += .025 * -abs(cos(posAng * PI / 180.0));
         }
      }
   }
   if (camState != 'n') {
      if (camState == 'l' && jDown) {
         camPos.x -= 0.25 * u.x;
         camPos.y -= 0.25 * u.y;
         camPos.z -= 0.25 * u.z;
         if (camPos.y < 0.0)
           camPos.y = 0.0;
         }
      else if (camState == 'r'&& lDown) {
         camPos.x += 0.25 * u.x;
         camPos.y += 0.25 * u.y;
         camPos.z += 0.25 * u.z;
         if (camPos.y < 0.0)
           camPos.y = 0.0;
      }
      else if (camState == 'f' && iDown) {
         camPos.x -= 0.25 * w.x;
         camPos.y -= 0.25 * w.y;
         camPos.z -= 0.25 * w.z;
         if (camPos.y < 0.0)
           camPos.y = 0.0;
      }
      else if (camState == 'b' && kDown) {
         camPos.x += 0.25 * w.x;
         camPos.y += 0.25 * w.y;
         camPos.z += 0.25 * w.z;
         if (camPos.y < 0.0)
           camPos.y = 0.0;
      }
   }
   if (posAng <= -360.0 || posAng >= 360.0)
      posAng = 0.0;
   curT.y = posAng;
   if (standPos.z >= 50.0)
      standPos.z = 50.0;
   else if (standPos.z <= -45.0)
      standPos.z = -45.0;
   if (standPos.x >= 45.0)
      standPos.x = 45.0;
   else if (standPos.x <= -50.0)
      standPos.x = -50.0;
   /*for (int i = 0; i < 20.0; i++) {
      if (sqrt(((standPos.x - treeTransX[i]) * (standPos.x - treeTransX[i])) + 
               ((standPos.z - treeTransZ[i]) * (standPos.z - treeTransZ[i]))) < treeRad)
         if (abs(treeTransX[i] - standPos.x) < abs(treeTransZ[i] - standPos.z)) {
            standPos.x = sqrt((treeRad * treeRad) - ((standPos.z - treeTransZ[i]) * (standPos.z - treeTransZ[i]))) + treeTransX[i];
         }
         else {
            standPos.z = sqrt((treeRad * treeRad) - ((standPos.x - treeTransX[i]) * (standPos.x - treeTransX[i]))) + treeTransZ[i];
         }
   }*/
   glutPostRedisplay();
}
